@@ -8,7 +8,6 @@
 
 #include "std_pair.hpp"
 
-
 inline const char *get_tag_by_key(osmium::TagList const& obj, const char *key)
 {
     if (!key) {
@@ -24,10 +23,16 @@ inline const char *get_tag_by_key(osmium::TagList const& obj, const char *key)
     return v;
 }
 
-inline bool taglist_contains_tag(osmium::TagList const& obj, const char *value)
+inline const char *get_tag_by_key_with_none(osmium::TagList const& obj,
+                                            const char *key)
 {
-    const char* v = obj.get_value_by_key(value);
-    return v;
+    return key ? obj.get_value_by_key(key) : nullptr;
+}
+
+
+inline bool taglist_contains_tag(osmium::TagList const& obj, const char *key)
+{
+    return key && obj.has_key(key);
 }
 
 inline const char member_item_type(osmium::RelationMember& obj)
@@ -56,9 +61,7 @@ BOOST_PYTHON_MODULE(_osm)
     docstring_options doc_options(true, true, false);
 
     to_python_converter<osmium::Timestamp, Timestamp_to_python>();
-    std_pair_to_python_converter<int, int>();
-    std_pair_to_python_converter<unsigned int, unsigned int>();
-    std_pair_to_python_converter<unsigned long, unsigned long>();
+    std_pair_to_python_converter<size_t, size_t>();
 
     enum_<osmium::osm_entity_bits::type>("osm_entity_bits")
         .value("NOTHING", osmium::osm_entity_bits::nothing)
@@ -72,7 +75,7 @@ BOOST_PYTHON_MODULE(_osm)
     ;
     class_<osmium::Location>("Location",
         "A geographic coordinate in WGS84 projection. A location doesn't "
-         "have to be necessarily valid.")
+         "necessarily have to be valid.")
         .def(init<double, double>())
         .add_property("x", &osmium::Location::x,
                       "(read-only) X coordinate (longitude) as a fixed-point integer.")
@@ -87,9 +90,10 @@ BOOST_PYTHON_MODULE(_osm)
                       "that it is within the usual bounds.")
     ;
     class_<osmium::Box>("Box",
-        "A bounding box around a geographic area. Such a box consists of two "
-        ":py:class:`osmium.osm.Location`s. Those locations may be invalid in "
-        "which case the box is considered invalid, too.")
+        "A bounding box around a geographic area. It is defined by an "
+        ":py:class:`osmium.osm.Location` for the bottem-left corner and an "
+        "``osmium.osm.Location`` for the top-right corner. Those locations may "
+        " be invalid in which case the box is considered invalid, too.")
         .def(init<double, double, double, double>())
         .def(init<osmium::Location, osmium::Location>())
         .add_property("bottom_left",
@@ -138,6 +142,9 @@ BOOST_PYTHON_MODULE(_osm)
         .def("__getitem__", &get_tag_by_key)
         .def("__contains__", &taglist_contains_tag)
         .def("__iter__", iterator<osmium::TagList,return_internal_reference<>>())
+        .def("get", &osmium::TagList::get_value_by_key,
+             (arg("self"), arg("key"), arg("default")))
+        .def("get", &get_tag_by_key_with_none)
     ;
     class_<osmium::NodeRef>("NodeRef",
         "A reference to a OSM node that also caches the nodes location.")
@@ -236,7 +243,7 @@ BOOST_PYTHON_MODULE(_osm)
              "Get the absolute value of the id of this object.")
         .def("user_is_anonymous", &osmium::OSMObject::user_is_anonymous,
              arg("self"),
-             "Check if the user anonymous. If true, the uid does not uniquely "
+             "Check if the user is anonymous. If true, the uid does not uniquely "
              "identify a single user but only the group of all anonymous users "
              "in general.")
     ;
@@ -275,6 +282,11 @@ BOOST_PYTHON_MODULE(_osm)
                       "(read-only) Ordered list of relation members. "
                       "See :py:class:`osmium.osm.RelationMemberList`")
     ;
+    class_<osmium::memory::ItemIteratorRange<osmium::InnerRing const> >("InnerRingIterator",
+            "Iterator over inner rings.",
+            no_init)
+        .def("__iter__", iterator<osmium::memory::ItemIteratorRange<osmium::InnerRing const>, return_internal_reference<> >())
+    ;
     class_<osmium::Area, bases<osmium::OSMObject>, boost::noncopyable>("Area",
             "Areas are a special kind of meta-object representing a polygon. "
             "They can either be derived from closed ways or from relations "
@@ -300,10 +312,8 @@ BOOST_PYTHON_MODULE(_osm)
                 &osmium::Area::cbegin<osmium::OuterRing>,
                 &osmium::Area::cend<osmium::OuterRing>),
              "Return an iterator over all outer rings of the multipolygon.")
-        .def("inner_rings", 
-              range<return_internal_reference<> >(
-                &osmium::Area::cbegin<osmium::InnerRing>,
-                &osmium::Area::cend<osmium::InnerRing>),
+        .def("inner_rings", &osmium::Area::inner_rings,
+                (arg("self"), arg("outer_ring")),
              "Return an iterator over all inner rings of the multipolygon.")
     ;
     class_<osmium::Changeset, boost::noncopyable>("Changeset",
@@ -317,7 +327,7 @@ BOOST_PYTHON_MODULE(_osm)
                       "(read-only) Timestamp when the changeset was first opened.")
         .add_property("closed_at", &osmium::Changeset::closed_at,
                       "(read-only) Timestamp when the changeset was finalized. May be "
-                      "None when the changeset is still open/")
+                      "None when the changeset is still open.")
         .add_property("open", &osmium::Changeset::open,
                       "(read-only) True when the changeset is still open.")
         .add_property("num_changes", &osmium::Changeset::num_changes,
